%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SPLINE.M

% This is a Matlab program, written by
% Bruce E. Hansen
% Department of Economics
% University of Wisconsin
% bhansen@ssc.wisc.edu
% http:/www.ssc.wisc.edu/~bhansen

% This program estimates the spline transfer function model presented in
% "How Responsive are Private Transfers to Income?
% Evidence From a Laissez-Faire Economy"
% by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.

% The results of this program generates the estimates reported in Table 3
% of the paper, and Figures 2 and 3.

% This program will take a long time to execute if one sets qn=0.  To speed up
% execution (with little cost in precision), set qn=500 or even qn=100.

% ***********************************************************/
function spline();
global sample;
global smp;
trm = .02;            % Percentile of high-income households deleted  @
per_ = .10;           % Percentage to trim from quantiles for threshold search @
qn   = 0;             % Set qn=0  to search across all thresholds.
                      % Set qn to a positive integers to search
                      % across a fixed number of quantiles     @
autop_= 0;           % If set to 1, will automatically print graphs   @
noinc_=1;             % If set to 1, includes dummy for no income, else set to 0 @
quad_=0;              % If set to 1, includes quadratic term for non-altruistic regime @
age2_=0;              % If set to 1, includes quadratic term in age of household head @
save_=0;              % Set to 1 to save transfer functions @

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
out=fopen('spline.txt','wt');
fclose(out);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
smp=1;
while smp<=2;  % Set to 1 if "urban" or 2 if "rural"  %
    rname=['Region1_';
        'Region2_';
        'Region3_';
        'Region4_';
        'Region5_';
        'Region6_';
        'Region7_';
        'Region8_';
        'Region9_';
        'Region10';
        'Region11';
        'Region12';
        'Region13'];
    vn=['__Abroad'                     
        'Domestic'                    
        '__Inkind'                     
        '___Gifts'                     
        '__Income'                     
        '___Wages'                     
        '____Self'                     
        '____Rent'                     
        'Interest'                     
        '__RetInc'                     
        'Dividend'                     
        '_Primary'                     
        'Some Hgh'                     
        'High Grd'                     
        'Some Col'                     
        '_College'                     
        '_____Age'                     
        '__Female'                     
        '_Married'                     
        '__Kids<1'                     
        '1<Kids<7'                     
        '7<Kid<15'                     
        '_HH Size'                     
        'Both Wrk'                     
        'Not Empl'                     
        '_Mar Fem'                     
        '__Region'];
    if smp==1;
        sample='Urban';
        rname=['Region0_';rname];
        knot=21840;
        load urban.txt;
        dat=urban;
    elseif smp==2;
        sample='Rural';
        knot=10793;
        load rural.txt;
        dat=rural;
    end;
    
    abroad=dat(:,1);
    domestic=dat(:,2);
    inkind=dat(:,3);
    gifts=dat(:,4);
    totinc=dat(:,5);
    retinc=dat(:,10);
    yname='Transfers';
    
    y = abroad+domestic+inkind-gifts;
    i = totinc+gifts-abroad-domestic-inkind-retinc;
    
    ind=0;
    ttempdat=[dat,i,y];
    for iii=1:length(i);
        if i(iii)>=0
            if ind==0
                tempdat=ttempdat(iii,:);
                ind=1;
            else
                tempdat=[tempdat;ttempdat(iii,:)];
            end;
        end;
    end;
    dat=tempdat;
    dat=sortrows(dat,28);
    n=floor(length(dat(:,1))*(1-trm));
    dat=dat(1:n,:);

    retinc=dat(:,10);
    i=dat(:,28);
    y=dat(:,29);
    dat=dat(:,1:27);
    
    dat(:,23)=dat(:,23)-dat(:,20)-dat(:,21)-dat(:,22);
    xx=[(retinc>0),retinc,dat(:,12:17)];
    xn=['Has_Retr';vn(10,:);vn(12:17,:)];
    if age2_==1
        xx=[xx,(dat(:,17).^2)];
        xn=[xn;'___Age^2'];
    end;
    region=dat(:,27);
    r=unique(region);
    xreg=(region*ones(1,length(r))==(ones(length(region),1)*r'));%???????%
    xx=[xx,dat(:,18:26),xreg];
    xn=[xn;vn(18:26,:);rname];
    mx=ones(n,1)*mean(xx);
    if noinc_==1
        i0=(i==0);
        xx=[i0,xx];
        xn=['NoIncome';xn];
    end;
    out=fopen('spline.txt','at');
    fprintf(out,'************************************************************\n\n\n');
    sh=strcat(sample,' households');
    fprintf(out,'%s\n',sh);
    fprintf(out,'Dependent Variable:   %s\n',yname);
    fprintf(out,'Trimmed Top Quantile of Income:  %f\n',trm);
    if noinc_==1
        fprintf(out,'Number of Zero Income:  %f\n',sum(i0));
    end;    
    fprintf(out,'Observations:                    %u\n',n);
    fprintf(out,'-------------------------------------------------\n');
    fclose(out);
    % Linear Model %
    x=[i,xx];
    beta=(y'/x')';
    e=y-x*beta;
    xname=['__Income';xn];
    out=fopen('spline.txt','at');
    fprintf(out,'Linear Model\n');
    report(y,x,e,beta,xname);
    fclose(out);
    
    % Polynomial Model %
    p=9;
    xp=(i*ones(1,p)).^(ones(length(i(:,1)),1)*(1:p))./(ones(length(i(:,1)),1)*(10000.^(0:p-1)));
    x=[xp,xx];
    beta=inv(x'*x)*(x'*y);
    e=y-x*beta;
    trp=xp*beta(1:p)+mx*beta(p+1+noinc_:length(beta(:,1)));
    
    xname='__Income';
    jp=2;
    while jp<=p
        xjp=strcat('Income^',num2str(jp));
        xname=[xname;xjp];
        jp=jp+1;
    end;
    xname=[xname;xn];
    out=fopen('spline.txt','at');
    fprintf(out,'Polynomial Model\n');
    report(y,x,e,beta,xname);
    fclose(out);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    xm=xx*inv(xx'*xx);
    e0=y-xm*(xx'*y);
    ee0=e0'*e0;
    q=i;
    if qn==0
        q=unique(q(floor(per_*n):(n-floor(per_*n))));
        qn=length(q(:,1));
    else
        q=q(floor((per_:(1-per_*2)/qn:per_+(qn-1)*((1-per_*2)/qn))*n));
    end;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Linear Spline Model %
    sse=zeros(qn,1);
    j=1;
    while j<=qn
        qj=q(j);
        ic=i-qj;
        i1=(i<=qj);
        i2=1-i1;
        dx=[(ic.*i1),(ic.*i2)];
        if quad_==1
            dx=[dx,(((ic.^2).*i2)/(10^6))];
        end;
        xv=dx-xm*(xx'*dx);
        et=e0-xv*(e0'/xv')';
        sse(j)=et'*et;
        j=j+1;
    end;
    [temp,jmin]=min(sse);
    knot=q(jmin);
    
    ic=i-knot;
    i1=(i<=knot);
    i2=1-i1;
    dx=[(ic.*i1),(ic.*i2)];
    if quad_==1
        dx=[dx,(((ic.^2).*i2)/(10^6))];
    end;
    x=[dx,xx];
    beta=inv(x'*x)*(x'*y);
    e=y-x*beta;
    xh=-beta(1).*i1-beta(2).*i2;
    if quad_==1
        xh=xh-(ic.*i2)*(2*beta(3));
    end;
    x=[x,xh];
    tr1=dx*beta(1:2)+mx*beta(3+noinc_:length(beta(:,1)));
    
    xname=['_Slope_1';'_Slope_2'];
    if quad_==1;
        xname=[xname;'__Quad_2'];
    end;
    xname=[xname;xn;'Threshol'];
    out=fopen('spline.txt','at');
    if quad_==0
        fprintf(out,'Linear Spline Model\n');
    else quad_==1
        fprintf(out,'Linear/Quadratic Spline Model\n');
    end;
    fprintf(out,'Number of Quantiles Used:    %u\n',qn);
    fprintf(out,'Income Threshold Quantile:   %f\n',mean(i<=knot));
    fclose(out);
    report(y,x,e,[beta;knot],xname);
    
    gr_tr(i,[trp,tr1]);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    smp=smp+1;
end;

function gr_tr(i,tr);
global sample;
global smp;
plot(i,tr(:,1),i,tr(:,2));
if smp==1
    xlim([0,200000]);
    ylim([0,20000]);
elseif smp==2
    xlim([0,80000]);
    ylim([0,10000]);
end;
sh2=strcat(sample,' households');
title(sh2);
xlabel('Total Income (Pesos)');
ylabel('Transfers (Pesos)');
saveas(gcf,strcat('spline',num2str(smp),'.eps'),'psc2');
grid on;

function report(y,x,e,theta,xname);
out=fopen('spline.txt','at');
ee=e'*e;
sig=ee/(length(y(:,1))-length(x(1,:)));
mi=inv(x'*x);
se=sqrt(abs(diag(mi*sig)));
seh=sqrt(abs(diag(mi*((x.*(e*ones(1,length(x(1,:)))))'*((x.*(e*ones(1,length(x(1,:)))))))*mi)));
fprintf(out,'Sum of Squared Errors:              %f\n',ee);
fprintf(out,'Residual Standard Deviation:        %f\n',sqrt(sig));
fprintf(out,'R-squared:                          %f\n\n',(1 - ee/(sum((y-mean(y)).^2))));
fprintf(out,'Variable    Estimate      St Error     Het St Er\n');
fprintf(out,'----------------------------------\n');
for iii=1:length(se)
    fprintf(out,'%s   %f   %f   %f\n',xname(iii,:),theta(iii),se(iii),seh(iii));
end;
fprintf(out,'\n\n\n\n');
    
